#include "qLab/qCommand.lsl"
#include "qLab/qPosition.lsl"

#define PRIM_COMMAND_CHANNEL 500

#define Q_PRIM_DELETESCRIPT    "-deleteScript"
#define Q_PRIM_SETOBJECTNAME   "-setObjectName"
#define Q_PRIM_SETOBJECTDESC   "-setObjectDesc"
#define Q_PRIM_SETTEXT         "-setText"
#define Q_PRIM_SLEEP           "-sleep"


list primCommand;

buildPrimCommand()
{
  primCommand = qCommandCreate("prim");
  primCommand = qCommandAddUniversalSwitches(primCommand);

  primCommand = qCommandAddSwitch(primCommand, [Q_PRIM_SETOBJECTNAME, 
                                        "string"]);
  primCommand = qCommandAddSwitch(primCommand, [Q_PRIM_SETOBJECTDESC, 
                                        "string"]);
  primCommand = qCommandAddSwitch(primCommand, [Q_PRIM_SETTEXT, 
                                        "string", "float", "float", "float", "float"]);
  primCommand = qCommandAddSwitch(primCommand, [Q_PRIM_SLEEP, 
                                        "integer"]);
  primCommand = qCommandAddSwitch(primCommand, [Q_PRIM_DELETESCRIPT]);

}


 
initPrimCommand()
{
  buildPrimCommand();

  llListen(PRIM_COMMAND_CHANNEL, "", NULL_KEY, "");

  integer privateChannel = llGetStartParameter();
  if (privateChannel != 0)
    llListen(privateChannel, "", NULL_KEY, "");
}


 
executePrimCommand(list switches)
{
  integer switchesLength = llGetListLength(switches);

  integer i;
  for (i = 1; i < switchesLength; )
    {
      string switch = llList2String(switches, i);

      if (switch == Q_PRIM_DELETESCRIPT)
	{
	  llRemoveInventory(llGetScriptName());
	  i++;
	}

      else if (switch == Q_PRIM_SETOBJECTNAME)
	{
	  llSetObjectName(llList2String(switches, i+1));
	  i+=2;
	}

      else if (switch == Q_PRIM_SETOBJECTDESC)
	{
	  llSetObjectDesc(llList2String(switches, i+1));
	  i+=2;
	}

      else if (switch == Q_PRIM_SETTEXT)
	{
	  llSetText(llList2String(switches, i+1),
		    (< llList2Float(switches, i+2), 
		     llList2Float(switches, i+3), 
		     llList2Float(switches, i+4) >),
		    llList2Float(switches, i+5));
	  i+=6;
	}

      else if (switch == Q_PRIM_SLEEP)
	{
	  llSleep(llList2Integer(switches, i+1));
	  i+=2;
	}


      else
	i = qCommandExecuteUniversalSwitch(switches, i);

    }
}

parsePrimCommand(string message)
{
 list switches = qCommandParseText(primCommand, message);

  if (switches != [])
    executePrimCommand(switches);

}


listenPrimCommand(integer channel, string name, key id,  string message)
{
  // we only accept commands on the public channel from our owner
  if ((channel == PRIM_COMMAND_CHANNEL) &&
      (name != llKey2Name(llGetOwner())))
    return;

  parsePrimCommand(message);
 }


linkMessagePrimCommand(integer sender, integer num, string message, key id)
{
  parsePrimCommand(message);
}


default
{
  state_entry()
    {
      initPrimCommand();
    }

  on_rez(integer n)
    {
      initPrimCommand();
    }

  listen(integer channel, string name, key id,  string message)
    {
      listenPrimCommand(channel, name, id, message);
    }

  link_message(integer sender, integer num, string message, key id)
    {
      linkMessagePrimCommand(sender, num, message, id);
    }
}
