#include "qLab/qCommand.lsl"
#include "qLab/qPosition.lsl"

#define SHAPE_COMMAND_CHANNEL 500

#define Q_SHAPE_SETTYPE         "-setType"
#define Q_SHAPE_DELETESCRIPT    "-deleteScript"
#define Q_SHAPE_SLEEP           "-sleep"


list shapeCommand;

buildShapeCommand()
{
  shapeCommand = qCommandCreate("shape");
  shapeCommand = qCommandAddUniversalSwitches(shapeCommand);

  shapeCommand = qCommandAddSwitch(shapeCommand, [Q_SHAPE_SETTYPE, 
                                        "integer"]);
  shapeCommand = qCommandAddSwitch(shapeCommand, [Q_SHAPE_SLEEP, 
                                        "integer"]);
  shapeCommand = qCommandAddSwitch(shapeCommand, [Q_SHAPE_DELETESCRIPT]);

}


 
initShapeCommand()
{
  buildShapeCommand();

  llListen(SHAPE_COMMAND_CHANNEL, "", NULL_KEY, "");

  integer privateChannel = llGetStartParameter();
  if (privateChannel != 0)
    llListen(privateChannel, "", NULL_KEY, "");
}


 
executeShapeCommand(list switches)
{
  integer switchesLength = llGetListLength(switches);

  integer i;
  for (i = 1; i < switchesLength; )
    {
      string switch = llList2String(switches, i);

      if (switch == Q_SHAPE_SETTYPE)
	{
	  integer type = llList2Integer(switches, i+1);
      
	  if (type == 0)
	    llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_BOX, 0, <0.0, 1.0, 0.0>, 
				  0.0, <0.0, 0.0, 0.0>, <1.0, 1.0, 0.0>, <0.0, 0.0, 0.0>]);
	  else if (type == 1)
	    llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_CYLINDER, 0, <0.0, 1.0, 0.0>, 0.0, 
				  <0.0, 0.0, 0.0>, <1.0, 1.0, 0.0>, <0.0, 0.0, 0.0>]);

	  else if (type == 2)
	    llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_PRISM, 0, <0.0, 1.0, 0.0>, 0.0, 
				  <0.0, 0.0, 0.0>, <0.0, 0.0, 0.0>, <0.0, 0.0, 0.0>]);

	  else if (type == 3)
	    llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_SPHERE, 0, <0.0, 1.0, 0.0>, 0.0, 
				  <0.0, 0.0, 0.0>, <0.0, 1.0, 0.0>]);

	  else if (type == 4)
	    llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_TORUS, 0, <0.0, 1.0, 0.0>, 0.0, 
				  <0.0, 0.0, 0.0>, <1.0, 0.25, 0.0>, <0.0, 0.0, 0.0>, 
				  <0.0, 1.0, 0.0>, <0.0, 0.0, 0.0>, 1.0, 0.0, 0.0]);

	  else if (type == 5)
	    llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_TUBE, 0, <0.0, 1.0, 0.0>, 0.0, 
				  <0.0, 0.0, 0.0>, <1.0, 0.25, 0.0>, <0.0, 0.0, 0.0>, 
				  <0.0, 1.0, 0.0>, <0.0, 0.0, 0.0>, 1.0, 0.0, 0.0]);

	  else if (type == 6)
	    llSetPrimitiveParams([PRIM_TYPE, PRIM_TYPE_RING, 0, <0.0, 1.0, 0.0>, 0.0, 
				  <0.0, 0.0, 0.0>, <1.0, 0.25, 0.0>, <0.0, 0.0, 0.0>, 
				  <0.0, 1.0, 0.0>, <0.0, 0.0, 0.0>, 1.0, 0.0, 0.0]);
                             
        

	  i+=2;
	}

      else if (switch == Q_SHAPE_DELETESCRIPT)
	{
	  llRemoveInventory(llGetScriptName());
	  i++;
	}


      else if (switch == Q_SHAPE_SLEEP)
	{
	  llSleep(llList2Integer(switches, i+1));
	  i+=2;
	}


      else
	i = qCommandExecuteUniversalSwitch(switches, i);

    }
}



parseShapeCommand(string message)
{
 list switches = qCommandParseText(shapeCommand, message);

  if (switches != [])
    executeShapeCommand(switches);

}


listenShapeCommand(integer channel, string name, key id,  string message)
{
  // we only accept commands on the public channel from our owner
  if ((channel == SHAPE_COMMAND_CHANNEL) &&
      (name != llKey2Name(llGetOwner())))
    return;

  parseShapeCommand(message);
 }


linkMessageShapeCommand(integer sender, integer num, string message, key id)
{
  parseShapeCommand(message);
}


default
{
  state_entry()
    {
      initShapeCommand();
    }

  on_rez(integer n)
    {
      initShapeCommand();
    }

  listen(integer channel, string name, key id,  string message)
    {
      listenShapeCommand(channel, name, id, message);
    }

  link_message(integer sender, integer num, string message, key id)
    {
      linkMessageShapeCommand(sender, num, message, id);
    }
}
